#!/usr/bin/python

import sys, time, random, itertools
import clique_mis_util as ut
from networkx import *



def get_minimal_cliques(D,G,h):
    minclique = set() 
    Gc = G.copy()
    while len(Gc)>0:
        u = ut.min_degree_vertex(Gc)
        Gu = Gc.subgraph(set(Gc.neighbors(u)).union([u]))
        minclique = minclique.union(ut.good_subsets2(set(Gc.neighbors(u)).union([u]),G,D,h))
        Gc.remove_node(u)
    return minclique

####main body###


#parse input arguments
path = sys.argv[1]
th = sys.argv[2]
ut.union_cite=True if sys.argv[3]=="u" else False 

#read input
D,hmax,hsingle,first,last = ut.cite_graph(path) #first argument: citation graph path
G = ut.comp_graph(path,th)       #second argument: threshold for comparability graph
ut.make_consistent(D,G)
ut.print_instance(D,G,hmax,hsingle)


#compute the h-index of unmerged profile
atomich = ut.current_h(D,hsingle)
finalh = 0

#try increasing values of h
for h in range(atomich+1,hmax+1):
    print "Trying h=",h 
    if ut.greedy_h(D,G,h,hsingle): continue
    #else enumerate the maximal cliques but remove edges incident with
    #good articles first
    Gc = G.copy()
    ut.clean_comp_graph(D,Gc,h)
    

    # #for each clique keep only minimal subsets with sufficient citations
    minc = get_minimal_cliques(D,Gc,h)
    print "Interesting minimal cliques:", len(minc)
    if (len(minc)+hsingle<h): 
            print "Not enough interesting minimal cliques", h-1
            finalh=h-1
            break
    else: #reduction to max independent set
        CG = ut.conf_graph(minc)
        ncg = len(CG)
        mcg = CG.size()
        ut.print_graph(CG,str(h)+".gra")
        print ncg, mcg
        if ncg-ut.greedy_vc(CG)+hsingle>=h: continue
        if ut.greedy_mis(CG)+hsingle>=h: 
            print "Greedy MIS successful"
            continue
        elif mcg>=float(ncg*ncg)/5.0: #try clique approach
            CCG = ut.conf_graph_compl(minc)
            print "Try Clique because CG is dense"
            print graph_clique_number(CCG)
            if graph_clique_number(CCG)+hsingle>=h: 
                finalh=h
                continue
            else: 
                print "Clique not big enough", h-1 
                finalh=h-1
                break
        elif ncg-ut.vc(CG)+hsingle<h: 
            print "Vertex Cover too big", h-1 
            finalh=h-1
            break

outf = open(sys.argv[4], 'a')

outf.write('{0},{1},{2},{3},{4},{5}\n'.format(path.split('/')[-1],first,last,th,str(atomich),str(finalh)))
