#!/usr/bin/python
from __future__ import print_function

import sys, time, random,math,itertools
from networkx import *
import networkx.algorithms.approximation

union_cite=True

def cite_graph(cfilename):
    D = DiGraph();
    hsingle =0;
    first = 1492;
    last = 2364;
    # Energy!
    for line in open(cfilename+".gra"):
        tabs = line.split();
        a = tabs[0]
        b = tabs[1].strip()
        D.add_edge(b,a)
    hmax = int(math.floor(math.sqrt(D.number_of_edges())));
    # print hmax
    for line in open(cfilename+".info"):
        if line.startswith("Only"):
            hmax = int(line.split()[5])
    hsingle = int(open(cfilename+".purged").readline())
    yearsf = open(cfilename+".years")
    first = yearsf.readline().strip()
    last = yearsf.readline().strip()
    return D,hmax,hsingle,first,last

def comp_graph(path,th):
    G = Graph();
    for line in open(path+"/"+th+".comp"):
        tabs = line.split();
        a = tabs[0]
        b = tabs[1].strip()
        G.add_edge(a,b)
    return G

def make_consistent(D,G):
    for a in G.nodes():
        if not a in D.nodes():
            G.remove_node(a)
    for a in D.nodes():
        if not a in G.nodes():
            G.add_node(a)

def print_instance(D,G,hmax,hsingle):
    print("Citation graph:",D.number_of_nodes(), "nodes", D.number_of_edges(), "cites")
    print("Compatibility graph:", G.number_of_nodes(), "nodes", G.number_of_edges(), "edges")
    print("Achievable h at most:", hmax)
    print("Atomic articles with sufficient cites:", hsingle)

def num_cites(merge,D):
    if union_cite:
        cites = set()
        for a in merge: cites = cites.union(D.predecessors(a))
        return len(cites)
    else: #sum cite
        cites=0
        for a in merge: cites += D.in_degree(a)
        return cites
        

def good_subsets(c,D,h):
    sol = set()
    for i in range(1,len(c)+1):
        ci = set(itertools.combinations(c, i))
        alle = True
        for cand in ci:
            if num_cites(cand,D)>=h: 
                safe_add(sol,cand)
            else: 
                alle = False
        if alle: break
    return sol

def good_subsets2(c,G,D,h):
    sol = set()
    for i in range(1,len(c)+1):
        ci = set(itertools.combinations(c, i))
        alle = True
        for cand in ci:
            if num_cites(cand,D)>=h and is_clique(cand,G): 
                safe_add(sol,cand)
            else: 
                alle = False
        if alle: break
    return sol


def current_h(D,hsingle):
    h=0
    better = True
    while(better):
        better = False
        ch = 0
        for v in D.nodes():
            if D.in_degree(v)>h:
                ch += 1
        if ch + hsingle> h:
            better = True
            h += 1
    print("Starting h-index:", h)
    return h

def safe_add(cset,c):
    contained=False
    for d in cset:
        if len(d)<len(c) and set(d)<=set(c):
            contained=True; break
    if not contained: 
        cset.add(c)
    
def conf_graph(cliques):
    G = Graph()
    for c in cliques:
        G.add_node(c)
    
    for u,v in set(itertools.combinations(cliques,2)):
        us=set(u) 
        vs=set(v) 
        if us.intersection(vs):
            G.add_edge(u,v)
    return G

def conf_graph_compl(cliques):
    G = Graph()
    for c in cliques:
        G.add_node(c)
    
    for u,v in set(itertools.combinations(cliques,2)):
        us=set(u) 
        vs=set(v) 
        if not us.intersection(vs):
            G.add_edge(u,v)
    return G


def vc(G): #return the size of a minimum vertex cover
    if G.number_of_edges()==0: return 0
    for k in range (1,G.number_of_nodes()):
        print(k)
        if recurse(G,k): return k

def recurse(G,k):
    if k<0: return False
    if G.number_of_edges()==0: 
        return True
    else:
        Gu = G.copy()
        #simple data reduction: remove simplicial vertices of degree at most four 
        for u in Gu.nodes():
            if Gu.degree(u)==1:
                Gu.remove_node(G.neighbors(u).pop())
                return recurse(Gu,k-1)
            if Gu.degree(u)==2:
                v,w= G.neighbors(u)
                if G.has_edge(v,w):
                    Gu.remove_nodes_from(G.neighbors(u))
                    return recurse(Gu,k-2)
            if Gu.degree(u)==3:
                v,w,x= G.neighbors(u)
                if G.has_edge(v,w) and G.has_edge(v,x) and G.has_edge(w,x):
                    Gu.remove_nodes_from(G.neighbors(u))
                    return recurse(Gu,k-3)
            if Gu.degree(u)==4:
                v,w,x,y= G.neighbors(u)
                if G.has_edge(v,y) and G.has_edge(w,y) and G.has_edge(x,y) and G.has_edge(v,w) and G.has_edge(v,x) and G.has_edge(w,x):
                    Gu.remove_nodes_from(G.neighbors(u))
                    return recurse(Gu,k-4)
        u = None
        #pick max degree vertex
        u = max_degree_vertex(G)
        #case 1: delete u
        Gu.remove_node(u)
        if recurse(Gu,k-1): return True 
        #case 2: delete N(u)
        Gv = G.copy()
        Gv.remove_nodes_from(G.neighbors(u))
        if recurse(Gv,k-len(G.neighbors(u))): return True
        return False #neither recursive call finds a solution 
        
def greedy_vc(Gc): #a greedy algorithm for computing small vertex covers 
    G = Gc.copy()
    k = 0
    while (G.number_of_edges()>0): 
        u = max_degree_vertex(G)
        k+=1
        G.remove_node(u)
    return k

def greedy_mis(Gc): #a greedy algorithm for computing large independent sets 
    G = Gc.copy()
    k = 0
    while (G.number_of_nodes()>0): 
        u = min_degree_vertex(G)
        k+= 1
        G.remove_nodes_from(G.neighbors(u))
        G.remove_node(u)
    return k


def max_degree_vertex(G): 
    u = None
    d = 0
    for v in G.nodes():
        if G.degree(v)>d:
            d = G.degree(v)
            u = v
    return u

def min_degree_vertex(G): 
    u = None
    d = G.number_of_nodes()
    for v in G.nodes():
        if G.degree(v)<d:
            d = G.degree(v)
            u = v
    return u

def greedy_h(D,G,h,hsingle):
    Gc =  G.copy()
    hleft = h-hsingle
    edges = Gc.edges()
    for (u,v) in edges:
        if num_cites([u,v],D)< h:
            Gc.remove_edge(u,v)
    while (Gc.size()>0):
        (u,v) =Gc.edges().pop()
        Gc.remove_node(u)
        Gc.remove_node(v)
        hleft -= 1
    return True if hleft <= 0 else False

def clean_comp_graph(D,G,h):
    for u in G.nodes():
        if num_cites([u],D)>=h:
            for v in G.neighbors(u):
                G.remove_edge(u,v)

def print_graph(G,fp):
    f = open(fp, 'w')
    f.write(str(G.number_of_nodes())+" "+str(G.number_of_edges()-G.number_of_selfloops())+"\n")
    idvert = dict()
    vertid = dict()
    i=1
    for u in G.nodes():
        idvert[i]=u
        vertid[u]=i
        i += 1
    for i in range(1,G.number_of_nodes()+1):
        neighbors = list()
        for v in G.neighbors(idvert[i]):
            neighbors.append(vertid[v])
        neighbors.sort()
        line = ""
        for v in neighbors: line = line+ str(v)+" "
        f.write(line.strip()+"\n")
    f.close()

def is_clique(s,G):
    is_clique = True
    uv = set(itertools.combinations(s, 2))
    for (u,v) in uv:
        if not G.has_edge(u,v): is_clique = False
    return is_clique
