import os
import sys
from split_index_exp import sum_cite, union_cite, read_graphs


def log(msg):
    sys.stderr.write(msg + '\n')
    sys.stderr.flush()


def h_index(partition, cite_graph, measure):
    profile = sorted([measure(x, cite_graph) for x in partition[:]],
                     reverse=True)
    i = 0
    while i < profile[i]:
        i += 1

    log("Counted in hindex" + str(profile[0:i]))

    return i


if __name__ == "__main__":
    if len(sys.argv) < 3:
        print("Usage: python3 path_to_data output.csv")

    with open(sys.argv[1], 'w') as outfile:
        outfile.write("dataset, author, unmerged_hindex_sumcite\n")

        path = sys.argv[1]
        if not os.path.isdir(path):
            raise RuntimeError("Target path is not a directory.")

        if path[-1] != '/':
            path += '/'

        for dataset in [name for name in os.listdir(path)
                        if os.path.isdir(path + name)]:
            for author in [name for name in os.listdir(path + dataset)
                           if os.path.isdir(path + dataset + '/' + name)]:

                G, D = read_graphs(dataset, author, "0.1")
                unmerged = [[x] for x in G.nodes()]
                unmerged_hindex_sumcite = h_index(unmerged, D, sum_cite)
                unmerged_hindex_unioncite = h_index(unmerged, D, union_cite)

                assert unmerged_hindex_sumcite == unmerged_hindex_unioncite
                # Note that sumcite, mergecite, fusion_cite should
                # give all the same values.

                outfile.write(",".join([dataset, author,
                                        str(unmerged_hindex_sumcite)])
                              + "\n")
