max.useful.k <- function(frame) {
    kres <- kmax <- max(frame$k)
    
    s <- frame[frame$k == kmax,]
    hinc <- s$hindex - s$original_hindex
    quants <- quantile(hinc)
    for(k in kmax:1) {
        s <- frame[frame$k == k,]
        hinc <- s$hindex - s$original_hindex
        quantsnew <- quantile(hinc)
        if(all(quantsnew == quants)) {
            kres <- k
        }
        else {
            return(kres)
        }
    }
    return(kres)
}

kmax <- function(frame) {
    res = max(frame$k)
    for(level in levels(frame$author)) {
        oneauthor <- frame[frame$author == level,]
        localmax <- max(oneauthor$k)
        if(localmax < res) {
            res <- localmax
        }
    }
    return(res)
}

ensure_everyone_has_c <- function(frame, co) {
    res <- frame
    for(level in levels(frame$author)) {
        filter <- frame[frame$author==level,]
        if(nrow(filter[filter$comp == co,])==0) {
            maxrow <- filter[which.max(filter$comp),]
            print(paste("adding dummy c", co, "for", level))
            maxrow$original_hindex <- maxrow$hsingle
            maxrow$hindex <- maxrow$hsingle
            maxrow$cliques <- 0
            maxrow$k <- 1
            maxrow$comp <- co
            res <- rbind(res, maxrow)
        }
    }
    return(res)
}

equalize_frame <- function(frame) {
    kmax <- max(frame$k)
    res <- frame
    for(level in levels(frame$author)) {
        filter <- frame[frame$author==level,]
        maxrow <- filter[which.max(filter$k),]
        if(maxrow$k < kmax) {
            for(i in seq(maxrow$k + 1,kmax)) {
                newrow <- maxrow
                newrow$k <- i
                res <- rbind(res, newrow)
            }
        }
    }
    return(res)
}

plot_with_k <- function(dir, alg, com) {
    files <- paste(dir, "/sum_cite_", alg, sep = "")
    fileu <- paste(dir, "/union_cite_", alg, sep="")
    
    s <- ensure_everyone_has_c(read.csv(files), com)
    u <- ensure_everyone_has_c(read.csv(fileu), com)

    scites <- equalize_frame(s[s$comp==com,])
    ucites <- equalize_frame(u[u$comp==com,])
    
    kmins <- min(scites$k)
    kmaxs <- max.useful.k(scites)

    kminu <- min(ucites$k)
    kmaxu <- max.useful.k(ucites)

    kmaxg <- max(kmaxu, kmaxs)

    xlimg <- max(kmaxg, 4)
    
    scites <- scites[scites$k <= kmaxg,]
    ucites <- ucites[ucites$k <= kmaxg,]
    
    hmax <- max(scites$hindex - scites$original_hindex, ucites$hindex - ucites$original_hindex)

    pdf(file=paste(dir, "/", alg, "-comp-", com, ".pdf", sep = ""), family="Times", width=8/2.54, height=5/2.54, pointsize=10)

    par(mar=c(2,2,0,0)+0.1, lwd=0.5, mgp=c(0,0.0,0), bty="n")


    boxplot((hindex-original_hindex)~k, data=scites, range=0, boxwex=0.25, at=kmins:kmaxg-0.15, yaxt="n", xaxt="n",ylim=c(0,hmax), xlim=c(0.75,kmaxg+0.75))
    boxplot((hindex-original_hindex)~k, data=ucites, range=0, boxwex=0.25, at=kminu:kmaxg+0.15, yaxt="n", xaxt="n",add=TRUE)

##    title(xlab="Number of allowed merges", ylab="H-index increase")

    axis(1,at=kmins:kmaxg,lwd=0.5,tck=0.03)
    axis(2,at=0:hmax,lwd=0.5,tck=0.03)
    dev.off()
    return(scites)
}

plot_without_k <- function(dir) {
    files <- paste(dir, "/sum_cite_extract", sep="")
    fileu <- paste(dir, "/union_cite_extract", sep="")

    scites <- read.csv(files)
    ucites <- read.csv(fileu)

    hmax <- max(scites$hindex - scites$original_hindex, ucites$hindex - ucites$original_hindex)

    pdf(file=paste(dir, "/extract.pdf", sep = ""), family="Times", width=8/2.54, height=5/2.54, pointsize=10)
    par(mar=c(2,2,0,0)+0.1, lwd=0.5, mgp=c(0,0.0,0), bty="n")


    boxplot((hindex-original_hindex)~comp, data=scites, range=0, boxwex=0.25, at=1:9-0.15, yaxt="n", xaxt="n",ylim=c(0,hmax))
    boxplot((hindex-original_hindex)~comp, data=ucites, range=0, boxwex=0.25, at=1:9+0.15, yaxt="n", xaxt="n",add=TRUE)

    axis(1,at=1:9,labels=seq(0.1,0.9,0.1),lwd=0.5,tck=0.03)
    axis(2,at=0:hmax,lwd=0.5,tck=0.03)

    dev.off()
    return(scites)
}

build_unbounded_frame <- function(frame) {
    res <- data.frame(author="",comp=0.1,hinc=0)[FALSE,]
    for(co in c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9)) {
        f <- ensure_everyone_has_c(frame, co)
        for(level in levels(f$author)) {
            oneauthor <- f[f$author == level & f$comp==co,]
            localmax <- max(oneauthor$hindex - oneauthor$original_hindex)
            res <- rbind(res, data.frame(author=level, comp=co, hinc=localmax))
        }
    }
    return(res)
}

plot_unbounded_k <- function(dir) {
    files <- paste(dir, "/sum_cite_cautious_atomizing", sep="")
    fileu <- paste(dir, "/union_cite_cautious_atomizing", sep="")

    scites <- build_unbounded_frame(read.csv(files))
    ucites <- build_unbounded_frame(read.csv(fileu))

    pdf(file=paste(dir, "/atomizing.pdf", sep = ""), family="Times", width=8/2.54, height=5/2.54, pointsize=10)
    par(mar=c(2,2,0,0)+0.1, lwd=0.5, mgp=c(0,0.0,0), bty="n")

    hmax <- max(scites$hinc, ucites$hinc)
    
    boxplot(hinc~comp, data=scites, range=0, boxwex=0.25, at=1:9-0.15, yaxt="n", xaxt="n",ylim=c(0,hmax))
    boxplot(hinc~comp, data=ucites, range=0, boxwex=0.25, at=1:9+0.15, yaxt="n", xaxt="n",add=TRUE)

    axis(1,at=1:9,labels=seq(0.1,0.9,0.1),lwd=0.5,tck=0.03)
    axis(2,at=0:hmax,lwd=0.5,tck=0.03)

    dev.off()
    return(scites)
    
}

make_k_plots <- function() {
    dirs <- paste("experiments/", c("ai10-2011", "ai10-2013", "ijcai-2013"), sep="")
    algos <- c("cautious_atomizing", "cautious_extract", "conservative_extract")
    merges <- c("halldor", "greedy_maxdeg", "greedy_mindeg", "maximum")
    for(dir in dirs) {
        plot_without_k(dir)
        plot_unbounded_k(dir)
        for(alg in algos) {
            for(merge in merges) {
                for(co in c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9)) {
                    plot_with_k(dir, alg, merge, co)
                }
            }
        }
    }
}
