## Plot for each dataset the number of profiles whose hindex can be
## improved by merges.

library(tikzDevice)
library(ggplot2)
library(tidyverse)

source("plots_util.R")

## Read csv
dat <- read.csv("exp.csv", 
                colClasses = c("factor", "factor",
                               "numeric", "factor",
                               "factor", "factor",
                               "integer", "integer",
                               "integer", "integer"))


## Nicer prints
dat <- as.tibble(dat)

dat <- latexlabels(dat)

## Filter changes, keep only one row per tuple of (author, measure,
## comp, merge_method)
did_change <- dat %>%
    group_by(measure, author, comp, merge_method) %>%
    filter(max_hindex > hmerged) %>%
    distinct(hmerged, .keep_all = TRUE) %>%
    ungroup()

## Plot

## summary <- did_change %>%
##     group_by(dataset, comp, measure, merge_method) %>%
##     summarize(n = n())

## write.csv(summary, "change_by_merging.csv")

## png("change_by_merging.png")
tikz("change_by_merging.tex", pointsize = 11, width = 4.8)

p <- ggplot(did_change) +
    geom_bar(mapping = aes(x = as.factor(10 * comp), fill = measure),
             position = "dodge",
             width = 0.9) +
    facet_grid(merge_method ~ dataset) +
    labs(x = "Compatibility threshold * 10",
         y = "Number of profiles",
         fill = "Measure") +
    theme(legend.title = element_blank(),
          legend.position = c(0.13, 0.915))

print(p)
dev.off()

