## Plot the initial h-index distributions after performing initial
## merges.

library(tikzDevice)
library(ggplot2)
library(tidyverse)

source("plots_util.R")

## Read csv
dat <- read.csv("exp.csv", 
                colClasses = c("factor", "factor",
                               "numeric", "factor",
                               "factor", "factor",
                               "integer", "integer",
                               "integer", "integer"))

## Nicer prints
dat <- as.tibble(dat)

dat <- latexlabels(dat)

## Keep only one row per tuple of (author, measure, comp, merge_method)
uniq_merged <- dat %>%
    group_by(measure, author, comp, merge_method) %>%
    distinct(hmerged, .keep_all = TRUE) %>% # There is only value of hmerged per tuple of (measure, author, comp, merge_method)
    ungroup()

## uniq_merged <- uniq_merged %>%
##     filter(comp == 0.4) %>%
##     filter(measure == "sum_cite")

uniq_selected <- uniq_merged %>%
    select(dataset, author, hmerged, measure, merge_method, comp)

## Prepare unmerged h-index data: Read, add dummy columns (and
## duplicate rows) for measure, merge_method, comp
unmerg_dat <- read.csv("unmerged_hindices.csv",
                       colClasses = c("factor", "factor", "integer"))
unmerg_dat <- as.tibble(unmerg_dat)

unmerg_dat$measure = "Unmerged"

grmaud <- mutate(unmerg_dat, merge_method = "GreedyMax")
grmiud <- mutate(unmerg_dat, merge_method = "GreedyMin")
ramsud <- mutate(unmerg_dat, merge_method = "Ramsey")
maxiud <- mutate(unmerg_dat, merge_method = "Maximum")

unmerg_dat <- rbind(grmaud, grmiud, ramsud, maxiud)

unmerg_dat$comp <- "U"

names(unmerg_dat) <- names(uniq_selected)

## Multiply compatibility threshold by 10 before binding; afterwards
## we cannot cast to numeric anymore.
uniq_selected <- uniq_selected %>%
    mutate(comp = as.factor(10 * as.numeric(comp))) 
plt_dat <- rbind(uniq_selected, unmerg_dat)

## Plot
tikz("initial_hindex_distributions.tex",
     pointsize = 11,
     width = 4.8,
     height = 6)

p <- ggplot(plt_dat) +
    geom_boxplot(coef = 0,
                 mapping = aes(x = comp,
                               y = hmerged,
                               fill = measure),
                 outlier.size = 0.3) +
    labs(x = "Compatibility threshold * 10",
         y = "H-index",
         fill = "Measure") +
    theme(legend.title = element_blank(),
          ## legend.position = c(0.7, 0.95),
          legend.position = "top") +
          ## legend.direction = "horizontal") + 
    facet_grid(merge_method ~ dataset)

print(p)
dev.off()
