library(tidyverse)

join_unmerged <- function(dat) {
    ## Join unmerged hindices into observations in dat.
    unmerg_dat <- read.csv("unmerged_hindices.csv",
                           colClasses = c("factor", "factor", "integer"))
    unmerg_dat <- as.tibble(unmerg_dat)
    uniq_add_unmerg <- left_join(dat, unmerg_dat, by = c("dataset", "author"))
    return(uniq_add_unmerg)
}

camelUpper <- function(x) {
    gsub("(^|[^[:alnum:]])([[:alnum:]])", "\\U\\2", x, perl = TRUE)
}

camelLower <- function(x) {
    y <- camelUpper(x)
    paste(tolower(substring(y, 1, 1)), substring(y, 2), sep = "")
}

## Latex-compatible labels and merge fusioncite problem names into
## union/sumcite problem names.
latexlabels <- function(x) {
    levels(x$measure) <- camelLower(levels(x$measure))
    x <- x %>%
        mutate(merge_method = fct_recode(merge_method,
                   "GreedyMin" = "greedy_mindeg",
                   "GreedyMax" = "greedy_maxdeg",
                   "Ramsey" = "halldor",
                   "Maximum" = "maximum"),
               problem = fct_recode(problem,
                   "Conservative Atomizing" = "conservative_atomizing",
                   "Conservative Atomizing" = "fusion_conservative_atomizing",
                   "Cautious Extracting" = "cautious_extracting",
                   "Cautious Extracting" = "fusion_cautious_extracting",
                   "Conservative Extracting" = "conservative_extracting",
                   "Conservative Extracting" = "fusion_conservative_extracting"))
    return(x)
}
                   
                   

                
