## Plot max hindex_increases vs compatibility thresholds
##

## setwd("~/proj/split-index/src")
library(ggplot2)
library(data.table)
library(tikzDevice)

source("plots_util.R")

## Read csv

dat <- read.csv("exp.csv", 
                colClasses = c("factor", "factor",
                               "numeric", "factor",
                               "factor", "factor",
                               "integer", "integer",
                               "integer", "integer"))

dat <- as.tibble(dat)

dat <- latexlabels(dat)

with_increase <- mutate(dat, hindex_increase = max_hindex - hmerged)

## Keep only those rows where the largest hindex_increase could be
## achieved for each author in each experiment, regardless of the
## unmerge operation (problem).
largest_increases <- with_increase %>%
    group_by(dataset, merge_method, author, comp, measure) %>%
    ## filter(hindex_increase == max(hindex_increase)) %>%
    ## This returns all maxima, we need exactly one. Use slice instead:
    slice(which.max(hindex_increase)) %>% 
    ungroup

only_potential <- filter(largest_increases, hindex_increase > 0)

tidy_dat <- only_potential

datasets = levels(tidy_dat$dataset)
merge_methods = levels(tidy_dat$merge_method)

## Individual plots
for (fdataset in datasets) {
    for (fmerge_method in merge_methods) {
        dat_plot <- tidy_dat %>%
            filter(dataset == fdataset,
                   merge_method == fmerge_method)

        plotname = paste("vs-comp", fdataset, fmerge_method, ".png", sep = "-")
        png(filename = plotname)
        p = ggplot(dat_plot,
                   aes(x = as.factor(comp), y = as.numeric(hindex_increase))) + 
            geom_boxplot(aes(fill = measure))
        print(p)
        dev.off()
    }
}

## Facet plot for greedy_maxdeg
dat_plot <- tidy_dat %>%
    filter(merge_method == "GreedyMax")

tikz("vs-comp-facet-greedy_maxdeg.tex", pointsize = 11, width = 4.8, height = 3)
## png(filename = "vs-comp-facet-greedy_maxdeg.png")

## jitter didn't work so nicely: still too many points.

## Not using whiskers below: useful behavior of whiskers not wanted by ggplot: https://github.com/tidyverse/ggplot2/issues/2522
p = ggplot(dat_plot) +
    geom_boxplot(coef = 0,
                 aes(x = as.factor(10 * comp),
                     y = as.numeric(hindex_increase),
                     fill = measure),
                     ## color = measure),
                 ## stat = "identity") +
                 ## U
                 outlier.size = 0.3) +
    ## guides(color = FALSE) +             # Remove legend for color aesthetic.
    labs(x = "Compatibility threshold * 10",
         y = "H-index increase",
         fill = "Measure") +
    theme(legend.title = element_blank(),
          legend.position = c(0.87, 0.77)) + 
    facet_wrap(~ dataset)

print(p)
dev.off()
